============
Contributing
============

Contributions are welcome, and they are greatly appreciated! Every little bit
helps, and credit will always be given.

You can contribute in many ways:

Types of Contributions
----------------------

Report Bugs
~~~~~~~~~~~

Report bugs at https://github.com/BNL-ATF/ophyd-basler/issues.

If you are reporting a bug, please include:

* Any details about your local setup that might be helpful in troubleshooting.
* Detailed steps to reproduce the bug.

Fix Bugs
~~~~~~~~

Look through the GitHub issues for bugs. Anything tagged with "bug" is open to
whoever wants to implement it.

Implement Features
~~~~~~~~~~~~~~~~~~

Look through the GitHub issues for features. Anything tagged with "feature" is
open to whoever wants to implement it.

Write Documentation
~~~~~~~~~~~~~~~~~~~

ophyd-basler could always use more documentation, whether as part of the
official ophyd-basler docs, in docstrings, or even on the web in blog posts,
articles, and such.

Submit Feedback
~~~~~~~~~~~~~~~

The best way to send feedback is to file an issue at
https://github.com/BNL-ATF/ophyd-basler/issues.

If you are proposing a feature:

* Explain in detail how it would work.
* Keep the scope as narrow as possible, to make it easier to implement.
* Remember that this is a volunteer-driven project, and that contributions are
  welcome :)

Get Started!
------------

Ready to contribute? Here's how to set up ``ophyd-basler`` for local
development.

1. Fork the ``ophyd-basler`` repo on GitHub or with ``gh repo fork
   BNL-ATF/ophyd-basler``.
2. Clone your fork locally::

    $ git clone https://github.com/your_name_here/ophyd-basler.git
    $ cd ophyd-basler/
    $ git remote add upstream https://github.com/BNL-ATF/ophyd-basler.git
    $ git pull --all

3. Install your local copy into a virtualenv or a conda environment.

   Assuming you have virtualenvwrapper installed, this is how you set up your
   fork for local development within a virtualenv::

    $ mkvirtualenv ophyd-basler
    $ python3 -m pip install -ve .

   or conda::

    $ conda create -n ophyd-basler -c conda-forge -y python=3.10
    $ conda activate ophyd-basler
    $ python3 -m pip install -ve .

4. Create a branch for local development::

    $ git checkout -b name-of-your-bugfix-or-feature

   Now you can make your changes locally.

5. When you're done making changes, check that your changes pass style checks
   and the tests::

    $ pip install -r requirements-dev.txt
    $ pre-commit
    $ pre-commit install
    $ pre-commit run --all-files
    $ pytest -s -vv

6. Commit your changes and push your branch to GitHub::

    $ git add .
    $ git commit -m "Your detailed description of your changes."
    $ git push -u origin name-of-your-bugfix-or-feature

7. Submit a pull request through the GitHub website.

Pull Request Guidelines
-----------------------

Before you submit a pull request, check that it meets these guidelines:

1. The pull request should include tests.
2. If the pull request adds functionality, the docs should be updated. Put your
   new functionality into a function with a docstring, and add the feature to
   the list in ``README.md``.
3. The pull request should work for Python 3.8+. Check the CI status from the
   ``README.md`` file and make sure that the tests pass for all supported
   Python versions.
